
# Require the two stdlib classes we need,
# which aren't included by default
require "net/http"
require "uri"
require "json"
require "openssl"

# The module namespace our classes will live in
module CyberArk
  # Custom exception class
  class InvalidURLError < StandardError
  end

  # Our main class definition
  class Credential

    # Custom getter method for @url class instance variable
    def base_url
      @base_url
    end

    # Custom setter method for @url class instance variable
    # because we're parsing the url parameter before assigning
    # to @url
    def base_url=(url)
      @base_url = url
    end


    def use_ssl
      @use_ssl
    end

    def use_ssl=(use_ssl)
      @use_ssl = use_ssl
    end


    # initialize method
    def initialize(url, use_ssl=false)

      @base_url = url
      @use_ssl = use_ssl

    end

    # Class method to make a GET request
    def get(app_id, query)
      begin
        # Use our @http_object object's request method to call the
        # Net::HTTP::Get class and return the resulting response object
        fullURL = "#{@base_url}/AIMWebService/api/Accounts?AppID=#{app_id}&Query=#{query}"
        parsedURI = URI.parse(fullURL)
        http_object = Net::HTTP.new(parsedURI.host, parsedURI.port)
        http_object.use_ssl = @use_ssl
        http_object.verify_mode = OpenSSL::SSL::VERIFY_NONE
        puts parsedURI.request_uri
        request = http_object.request(Net::HTTP::Get.new(parsedURI.request_uri))
        result = JSON.parse(request.body)
	if result.key?("ErrorCode")
	   raise InvalidURLError.new("#{result['ErrorCode']}-#{result['ErrorMsg']}")
	end
	result
      rescue URI::InvalidURIError
        # Raise a custom exception with a more friendly message
        raise InvalidURLError.new("#{fullURL} was not a valid URL.")
      end
    end
  end
end

#puts "initializing"
#http_requestor = CyberArk::Credential.new("https://192.168.86.162", use_ssl=true)
#puts "performing GET request"
#puts "Response code was #{http_requestor.get('Chef_App', query='Safe=Test;Folder=Root;Object=Object')['Content']}"

